/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 08/06/22.
//

#ifndef FGGL_DEBUG_IMPL_LOGGING_SPDLOG_HPP
#define FGGL_DEBUG_IMPL_LOGGING_SPDLOG_HPP

#include <string>
#include <iostream>
#include <memory>

#include <spdlog/spdlog.h>
#include <spdlog/sinks/stdout_color_sinks.h>

namespace fggl::debug {

	using FmtType = const std::string_view;

	/**
	 * Logging levels
	 */
	enum class Level {
		critical = spdlog::level::critical,
		error = spdlog::level::err,
		warning = spdlog::level::warn,
		info = spdlog::level::info,
		debug = spdlog::level::debug,
		trace = spdlog::level::trace
	};

	template<typename ...T>
	void error(const FmtType &fmt, T &&...args) {
		spdlog::error(fmt, args...);
	}

	template<typename ...T>
	void warning(const FmtType &fmt, T &&...args) {
		spdlog::warn(fmt, args...);
	}

	template<typename ...T>
	void info(const FmtType &fmt, T &&...args) {
		spdlog::info(fmt, args...);
	}

	template<typename ...T>
	void debug(const FmtType &fmt, T &&...args) {
		spdlog::debug(fmt, args...);
	}

	template<typename ...T>
	void trace(const FmtType &fmt, T &&...args) {
		spdlog::trace(fmt, args...);
	}

	template<typename ...T>
	void log(const FmtType &fmt, T &&...args) {
		spdlog::log(Level::info, fmt, args...);
	}

	template<typename ...T>
	void log(Level level, const FmtType &fmt, T &&...args) {
		spdlog::log(level, fmt, args...);
	}

	class Logger {
		public:
			Logger();

			template<typename ...Args>
			void log(Level level, const FmtType &fmt, Args &&...args) {
				spdlog::log(level, fmt, args...);
			}
	};

}

#endif //FGGL_DEBUG_IMPL_LOGGING_SPDLOG_HPP
